unit Frame;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DropSource, DropTarget, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Edit1: TEdit;
    Edit2: TEdit;
    DropTextTarget1: TDropTextTarget;
    DropTextTarget2: TDropTextTarget;
    Label1: TLabel;
    Label2: TLabel;
    DropTextSource1: TDropTextSource;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure DropTextTarget1Drop(Sender: TObject; ShiftState: TShiftState;
      Point: TPoint; var Effect: Integer);
    procedure DropTextTarget2Drop(Sender: TObject; ShiftState: TShiftState;
      Point: TPoint; var Effect: Integer);
    procedure FormShow(Sender: TObject);
    procedure Edit1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Edit1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure Edit2MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    { Private 錾 }
    // used by top example
    DragPoint: TPoint;

    // used by bottom example
    OldEdit2WindowProc: TWndMethod;
    procedure NewEdit2WindowProc(var Msg: TMessage);

    function MouseIsOverEdit2Selection(XPos: Integer): Boolean;
    procedure StartEdit2Drag;
  public
    { Public 錾 }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
  DropTextTarget1.Register(Edit1);
  DropTextTarget2.Register(Edit2);

  // Used for Bottom Text Drag example...
  // Hook edit window so we can intercept WM_LBUTTONDOWN messages!
  OldEdit2WindowProc := Edit2.WindowProc;
  Edit2.WindowProc := NewEdit2WindowProc;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  DropTextTarget2.UnregisterAll;
  DropTextTarget1.UnregisterAll;

  // Used by Bottom Text Drag example...
  // Undo hooking...
  Edit2.WindowProc := OldEdit2WindowProc;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  Label1.Caption :=
  'ȉ̃eLXghbOAhhbvT|[g鑼̃AvP[V' + #13#10 + #13#10 +
  '([vAGfB^[)փhbOAhhbvČĉB' + #13#10 + #13#10 +
  '܂tɑ̃AvP[V牽eLXgȉ̕ҏW{bNX' + #13#10 + #13#10 +
  'hbOAhhbvČĉB';

  Label2.Caption :=
  'ȉ̃eLXg𕔕IɑIăhbOAhhbvT|[g' + #13#10 + #13#10 +
  '̃AvP[V([vAGfB^[)փhbOAhhbv' + #13#10 + #13#10 +
  'ČĉB܂tɑ̃AvP[V牽eLXgȉ' + #13#10 + #13#10 +
  'ҏW{bNXɃhbOAhhbvČĉB';
end;

procedure TForm1.Edit1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  DragPoint := Point(X, Y);
end;

procedure TForm1.Edit1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  Edit1.SelLength := 0;

  if (Shift <> [ssLeft]) or (Edit1.Text = '') then
    Exit;
  // Make sure mouse has moved at least 10 pixels before starting drag ...
  if (Abs(DragPoint.X - X) < Mouse.DragThreshold) and
     (Abs(DragPoint.Y - Y) <Mouse.DragThreshold) then
    Exit;

  DropTextTarget1.DragTypes := []; //Disable Edit1 as a target temporarily.

  DropTextSource1.Text := Edit1.Text;
//--------------------------
  DropTextSource1.Execute; // Do it here - dragging from Edit1
//--------------------------

  DropTextTarget1.DragTypes := [dtCopy, dtMove, dtLink]; // Enable Edit1 as a drop target again.
end;

procedure TForm1.DropTextTarget1Drop(Sender: TObject;
  ShiftState: TShiftState; Point: TPoint; var Effect: Integer);
begin
  Edit1.Text := DropTextTarget1.Text; // Dragging to Edit1.
end;

// The new WindowProc for Edit2 which intercepts WM_LBUTTONDOWN messages
// before ANY OTHER processing...
procedure TForm1.NewEdit2WindowProc(var Msg: TMessage);
begin
  with TWMMouse(Msg) do
    if (Msg = WM_LBUTTONDOWN) and MouseIsOverEdit2Selection(XPos) then
    begin
      StartEdit2Drag; // Just a locally declared procedure.
      Result := 0;
      Exit;
    end;
  // Otherwise do everything as before...
  OldEdit2WindowProc(Msg);
end;

function TForm1.MouseIsOverEdit2Selection(XPos: Integer): Boolean;
var
  DC: HDC;
  SavedFont: HFont;
  Size1, Size2: TSize;
  S1, S2: String;
begin
  with Edit2 do
  begin
    Result := False;
    if (SelLength = 0) or (not Focused) then
      Exit;

    DC := GetDC(0);
    SavedFont := SelectObject(DC, Font.Handle);
    S1 := Copy(Text, 1, SelStart);
    S2 := S1 + SelText;
    GetTextExtentPoint32(DC, PChar(S1), Length(S1), Size1);
    GetTextExtentPoint32(DC, PChar(S2), Length(S2), Size2);
    SelectObject(DC, SavedFont);
    ReleaseDC(0, DC);
    if (XPos >= Size1.cx) and (XPos <= Size2.cx) then
      Result := True;
  end;
end;

procedure TForm1.StartEdit2Drag;
begin
  DropTextSource1.Text := Edit2.SelText;
//--------------------------
  DropTextSource1.Execute; // Do it here!!!!!
//--------------------------
end;

//This method just changes mouse cursor to crHandPoint if over selected text...
procedure TForm1.Edit2MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  with Edit2 do
    if MouseIsOverEdit2Selection(X) then
      Cursor := crHandPoint
    else
      Cursor := crDefault;
end;

procedure TForm1.DropTextTarget2Drop(Sender: TObject;
  ShiftState: TShiftState; Point: TPoint; var Effect: Integer);
begin
  Edit2.SelText := DropTextTarget2.Text; //Dragging to Edit2.
end;

end.
